package com.idiot.operationbackend.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.idiot.operationbackend.entity.AccountFansStat;
import com.idiot.operationbackend.mappers.AccountFansStatMapper;
import com.idiot.operationbackend.service.facade.AccountFansStatService;
import com.idiot.operationbackend.service.facade.FansActionStatService;
import com.idiot.operationbackend.service.facade.WeChatService;
import com.idiot.operationbackend.support.Constants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.util.List;

/**
 * 公众号数据统计
 * @author wang xiao
 * @date Created in 14:54 2020/9/15
 */
@Service
public class AccountFansStatServiceImpl extends ServiceImpl<AccountFansStatMapper, AccountFansStat>
        implements AccountFansStatService {



    @Autowired
    private WeChatService weChatService;


    @Autowired
    private FansActionStatService actionStatService;


    @Override
    public List<AccountFansStat> queryByDateAndAccIds(List<String> accountIds, String statDate) {
        return list(Wrappers.<AccountFansStat>lambdaQuery().eq(AccountFansStat::getStatDate,statDate)
                .in(AccountFansStat::getAccountId,accountIds));
    }


    @Override
    public AccountFansStat queryByDateAndAccountId(String accountId, String statDate) {
        return getOne(Wrappers.<AccountFansStat>lambdaQuery()
                .eq(AccountFansStat::getAccountId,accountId)
                .eq(AccountFansStat::getStatDate,statDate));
    }


    @Override
    @Async("asyncExecutor")
    public void statAccountFansData(String accountId) {
        String yesterday = Constants.DATE_FORMATTER.format(LocalDate.now().plusDays(-1));
        String beforeDay = Constants.DATE_FORMATTER.format(LocalDate.now().plusDays(-2));
        String jsonStr = weChatService.getFansSunmmary(accountId,yesterday,yesterday);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        JSONArray jsonArray = jsonObject.getJSONArray("list");
        int size  = jsonArray.size();
        int newNum = 0;
        int cancelNum = 0;
        int addNum = 0;
        JSONObject tempObject = null;
        // 昨日增减
        for (int i = 0; i < size; i++) {
            tempObject = jsonArray.getJSONObject(i);
            newNum += tempObject.getInteger("new_user");
            cancelNum += tempObject.getInteger("cancel_user");
        }
        int totalNum = 0;
        // 昨日汇总
        String jsonStrTotal = weChatService.getFansCumulate(accountId,yesterday,yesterday);
        JSONObject jsonObjectTotal = JSONObject.parseObject(jsonStrTotal);
        JSONArray jsonArrayTotal = jsonObjectTotal.getJSONArray("list");
        int sizeTotal  = jsonArrayTotal.size();
        for (int i = 0; i < sizeTotal; i++) {
            tempObject = jsonArray.getJSONObject(i);
            totalNum += tempObject.getInteger("cumulate_user");
        }
        // 昨日和前日 数据
        List<AccountFansStat> statData = baseMapper.selectByDateAndAccId(accountId,yesterday,beforeDay);
        // 昨日活跃数量
        int inactiveNum =  inactiveFansNum(accountId,yesterday);
        AccountFansStat ydData = statData.stream().filter(e->yesterday.equals(e.getStatDate())).findFirst().orElseGet(AccountFansStat::new);
        addNum = newNum-cancelNum;
        ydData.setTotalFansNum(totalNum);
        ydData.setNewNum(newNum);
        ydData.setCancelNum(cancelNum);
        ydData.setStatDate(yesterday);
        ydData.setAccountId(accountId);
        ydData.setAddNum(addNum);
        ydData.setInactiveNum(inactiveNum);

        statData.stream().filter(e->beforeDay.equals(e.getStatDate())).findFirst().map(e->{
            ydData.setAddRate(Constants.calcRate(ydData.getAddNum(), e.getAddNum()));
            ydData.setNewRate(Constants.calcRate(ydData.getNewNum(), e.getNewNum()));
            ydData.setCancelRate(Constants.calcRate(ydData.getCancelNum(), e.getCancelNum()));
            ydData.setInactiveRate(Constants.calcRate(ydData.getInactiveNum(), e.getInactiveNum()));
            ydData.setPageReadRate(Constants.calcRate(ydData.getPageReadNum(),e.getPageReadNum()));
            ydData.setInactiveRate(Constants.calcRate(ydData.getInactiveNum(),e.getInactiveNum()));
            return e;
        });
        saveOrUpdate(ydData);
    }


    @Override
    public int inactiveFansNum(String accountId, String statDate) {
        LocalDate localDate = LocalDate.parse(statDate,Constants.DATE_FORMATTER);
        long start =  localDate.atTime(LocalTime.of(0,0,0)).toInstant(Constants.DEFAULT_ZONE).getEpochSecond();
        long end = localDate.atTime(LocalTime.of(23,59,59)).toInstant(Constants.DEFAULT_ZONE).getEpochSecond();
        return actionStatService.countInactiveFansNum(accountId,start,end);
    }



}
