package com.idiot.operationbackend.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.idiot.operationbackend.entity.Account;
import com.idiot.operationbackend.entity.AccountTag;
import com.idiot.operationbackend.mappers.AccountTagMapper;
import com.idiot.operationbackend.service.facade.AccountTagService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


/**
 * @author wang xiao
 */

@Service
public class AccountTagServiceImpl  extends ServiceImpl<AccountTagMapper,AccountTag>
        implements AccountTagService {

    private final Logger logger = LoggerFactory.getLogger(AccountTagServiceImpl.class);


    @Override
    public boolean overlayAccountTag(String accountId, List<AccountTag> newTags) {
        logger.info("同步粉丝标签 保存/删除数据库 --------start");
        List<AccountTag> preData = queryAccountTag(accountId);
        List<AccountTag> addTag = newTags.stream().filter(e->!ifHave(preData,e.getWxId())).collect(Collectors.toList());
        List<AccountTag> delTag = preData.stream().filter(e->!ifHave(newTags,e.getWxId())).collect(Collectors.toList());
        boolean ifAdd = addAccountTag(addTag);
        boolean ifDel = delAccountTag(delTag);
        logger.info("同步粉丝标签 保存/删除数据库 ---------end,add 结果：{},del 结果：{}",ifAdd,ifDel);
        return ifAdd & ifDel;
    }


    @Override
    public List<AccountTag> queryAccountTag(String accountId) {
        return list(Wrappers.<AccountTag>lambdaQuery().eq(AccountTag::getAccountId,accountId));
    }


    @Override
    public boolean addAccountTag(List<AccountTag> accountTags) {
        if (CollectionUtils.isEmpty(accountTags)) {
            return false;
        }
        return saveBatch(accountTags);
    }

    @Override
    public boolean delAccountTag(List<AccountTag> accountTags) {
        if (CollectionUtils.isEmpty(accountTags)) {
            return false;
        }
        List<String> ids = accountTags.stream().map(AccountTag::getId).collect(Collectors.toList());
        return removeByIds(ids);
    }

    private boolean ifHave(List<AccountTag> accountTags, Integer wxId) {
        if (CollectionUtils.isEmpty(accountTags)) {
            return false;
        }
        return accountTags.stream().anyMatch(e->wxId.equals(e.getWxId()));
    }




}
